/**
 *
 */
package abc;

import java.io.IOException;
import java.io.LineNumberReader;

import abc.parser.Parser;
import abc.score.Score;
import abc.translator.Translator;
import util.Debug;
import util.Option;

/**
 * ABCからMoE-ABCへと変換する
 *
 * @author CrapeMyrtle
 *
 */
public class Abc2MoeAbc {
	/**
	 * 変換を実施する
	 *
	 * @param opt
	 *            オプション
	 * @throws IOException
	 *             リード中に発生した例外
	 */
	public void convert(Option opt) throws IOException {
		Parser parser = new Parser(new LineNumberReader(opt.getIn()));
		Score score = parser.parse(opt);
		parser = null;

		if (opt.getError() != null) {
			throw new RuntimeException(opt.getError()
					+ "\nヘルプ(-h,--help)、もしくは詳細なヘルプ(-h,--help-detail)を参照下さい");
		}
		if (opt.isHelp() || opt.isHelpDetail()) {
			System.out.print(opt.help());
			return;
		}

		Debug.out(opt);

		Debug.out(">>> Parsing End <<<");
		Debug.out(score);

		Translator translator = new Translator(opt);

		translator.optimize(score, false); // 繰り返しを展開するために最適化
		translator.expandRepeat(score);
		translator.optimize(score, false);
		translator.updateGrace(score);
		translator.optimize(score, false);
		translator.combineTie(score);
		translator.optimize(score, false);
		translator.updateDecoration(score);
		translator.combineVoiceOverlay(score);
		translator.copyInformation(score);
		translator.optimize(score, true);
		if (opt.isFixBugTempoChange()) {
			translator.fixTempoChange(score);
		}
		translator.addTie(score);
		translator.optimizeVolume(score);
		translator.optimizeVelocity(score);
		translator.optimize(score, true); // 最終的な最適化

		Debug.out(">>> Optimization End <<<");
		Debug.out(score);
		translator.translate2MoeAbc(score);
	}
}
