/**
 *
 */
package abc.parser;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import abc.score.InformationFieldClef;

/**
 * clefオプションのパーサ
 *
 * @author CrapeMyrtle
 *
 */
public class ParserInformationFieldClef {
	private static final Pattern	PTN_CLEF_NAME	= Pattern.compile("\\s*(?:clef\\s*=)?\\s*(treble|alto|tenor|bass|perc|none)([\\+\\-]8)?");
	private static final Pattern	PTN_OCTAVE		= Pattern.compile("\\s*octave\\s*=\\s*([\\+\\-]?[0-9]+)");
	private static final Pattern	PTN_TRANSPOSE	= Pattern.compile("\\s*t(?:ranspose)?\\s*=\\s*([\\+\\-]?[0-9]+)");

	/**
	 * clefオプションを解析する
	 *
	 * @param str
	 *            解析する文字列
	 * @return 解析結果
	 */
	public InformationFieldClef parse(String str) {
		InformationFieldClef clef = new InformationFieldClef();
		Matcher m;

		m = PTN_CLEF_NAME.matcher(str);
		if (m.find()) {
			// clef nameが指定されていたらオクターブ情報はリセットする
			clef.setClefOctave(0);
			clef.setOctave(0);

			if (m.group(2) != null) {
				clef.setClefOctave(Integer.parseInt(m.group(2)) / 8);
			}
			str = m.replaceFirst("");
		}

		m = PTN_OCTAVE.matcher(str);
		if (m.find()) {
			clef.setOctave(Integer.parseInt(m.group(1)));
			str = m.replaceFirst("");
		}

		m = PTN_TRANSPOSE.matcher(str);
		if (m.find()) {
			clef.setTransposition(Integer.parseInt(m.group(1)));
			str = m.replaceFirst("");
		}

		return clef;
	}
}
