/**
 *
 */
package abc.parser;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import abc.score.EKeySignature;
import abc.score.InformationFieldClef;
import abc.score.InformationFieldKey;
import abc.score.Key;

/**
 * information fieldにあるK:フィールドのパーサ
 *
 * @author CrapeMyrtle
 *
 */
public class ParserInformationFieldKey {
	private static final Pattern	PTN_KEY		= Pattern
														.compile("(?:^\\s*|\\s+)([A-G][b#]?)\\s*"
																+ "(?i)(?:(maj|min|m|ion|aeo|mix|dor|phr|lyd|loc)[a-z]*)?");
	private static final Pattern	PTN_EXP		= Pattern
														.compile("(?i)(?:^\\s*|\\s+)(exp)[a-z]*");
	private static final Pattern	PTN_EXP_KEY	= Pattern
														.compile("(?i)(?<=^\\s*|\\s+|[\\^_=][a-g])([\\^_=])([a-g])(?=[\\^_=][a-g]|$)");

	/**
	 * 文字列を解析する
	 *
	 * @param str
	 *            解析する文字列
	 * @return information fieldのK:フィールド
	 */
	public InformationFieldKey parse(String str) {
		// TODO 結構適当なのでうまくいかないものもあるかも
		// TODO transposeなども未実装。あんまりやる気なし。

		Key key = new Key();

		Matcher m = PTN_KEY.matcher(str);
		if (m.find()) {
			String base = m.group(1);
			String mode = m.group(2);

			if (mode == null) {
				mode = "";
			}

			key.setKey(base, mode);
			str = m.replaceFirst("");
		}

		m = PTN_EXP.matcher(str);
		if (m.find()) {
			// exp 以降のキーで上書きするため、全部ナチュラルにしておけば良い？
			String abc = "ABCDEFG";
			for (int i = 0; i < abc.length(); ++i) {
				key.setKeySign(abc.substring(i, i + 1), EKeySignature.NATURAL);
			}

			str = m.replaceFirst("");
		}

		while (true) {
			m = PTN_EXP_KEY.matcher(str);
			if (m.find()) {
				EKeySignature sign = null;
				switch (m.group(1)) {
				case "^":
					sign = EKeySignature.SHARP;
					break;
				case "=":
					sign = EKeySignature.NATURAL;
					break;
				case "_":
					sign = EKeySignature.FLAT;
					break;
				}
				key.setKeySign(m.group(2), sign);

				str = m.replaceFirst("");
			} else {
				break;
			}
		}

		InformationFieldClef clef = new ParserInformationFieldClef().parse(str);

		return new InformationFieldKey(key, clef);
	}
}
