/**
 *
 */
package abc.parser;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import abc.score.InformationFieldTempo;
import abc.score.Tempo;
import abc.util.Fraction;

/**
 * information fieldにあるK:フィールドのパーサ
 *
 * @author CrapeMyrtle
 *
 */
public class ParserInformationFieldTempo {

	private static final Pattern	PTN_TEXT		= Pattern.compile("\"" + Constant.STR_STRING
															+ "\"");
	private static final Pattern	PTN_TEMPO1		= Pattern.compile("(C|\\d+\\s*/\\s*\\d+(?:\\s+\\d+\\s*/\\s*\\d+)*)\\s*=\\s*(\\d+)");
	private static final Pattern	PTN_TEMPO2		= Pattern.compile("\\d+");
	private static final Pattern	PTN_TEMPO_BASE	= Pattern.compile("\\d+\\s*/\\s*\\d+");

	/**
	 * 文字列を解析する
	 *
	 * @param str
	 *            解析する文字列
	 * @return information fieldのQ:フィールド
	 */
	public InformationFieldTempo parse(String str) {
		// TODO 結構適当なのでうまくいかないものもあるかも

		Tempo tempo = new Tempo();
		String text = "";

		Matcher m = PTN_TEXT.matcher(str);
		if (m.find()) {
			text = m.group(1);

			str = m.replaceAll("");
		}

		m = PTN_TEMPO1.matcher(str);
		if (m.find()) {
			tempo.setCount(Integer.parseInt(m.group(2)));

			str = m.group(1);
			if (str.indexOf("C") < 0) {
				Fraction base = new Fraction(0);
				while (true) {
					m = PTN_TEMPO_BASE.matcher(str);
					if (!m.find()) {
						break;
					}

					base.add(m.group(0).indexOf("C") >= 0
							? new Fraction(1)
							: Fraction.parseFraction(m.group(0)));

					str = str.substring(m.end());
				}
				tempo.setBase(base);
			}
		} else {
			m = PTN_TEMPO2.matcher(str);
			if (m.find()) {
				tempo.setCount(Integer.parseInt(m.group(0)));
			}
		}

		return new InformationFieldTempo(tempo, text);
	}
}
