/**
 *
 */
package abc.score;

import java.util.LinkedList;

/**
 * 前打音を表すクラス
 *
 * @author CrapeMyrtle
 *
 */
public class GraceNote implements Cloneable {
	private boolean				isShortGrace	= false;
	private LinkedList<Note>	notes			= new LinkedList<Note>();

	/**
	 * コンストラクタ
	 */
	public GraceNote() {
	}

	/**
	 * コンストラクタ
	 *
	 * @param notes
	 *            Grace Noteを構成する音符
	 * @param isShortGrace
	 *            短前打音ならばtrue
	 */
	public GraceNote(LinkedList<Note> notes, boolean isShortGrace) {
		this.notes = notes;
		this.isShortGrace = isShortGrace;
	}

	/*
	 * (非 Javadoc)
	 *
	 * @see java.lang.Object#clone()
	 */
	public GraceNote clone() {
		LinkedList<Note> notes = new LinkedList<Note>();
		for (Note note : this.notes) {
			notes.add(note.clone());
		}

		return new GraceNote(notes, this.isShortGrace);
	}

	/*
	 * (非 Javadoc)
	 *
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return (this.isShortGrace ? "/" : "") + notes.toString();
	}

	/**
	 * 短前打音かどうかを取得する。
	 *
	 * @return true:短前打音。false:長前打音。
	 */
	public boolean isShortGrace() {
		return isShortGrace;
	}

	/**
	 * 短前打音かどうかをセットする。
	 *
	 * @param isShortGrace
	 *            true:短前打音。false:長前打音。
	 */
	public void setShortGrace(boolean isShortGrace) {
		this.isShortGrace = isShortGrace;
	}

	/**
	 * 前打音を構成する音符リストを取得する。
	 *
	 * @return 音符リスト
	 */
	public LinkedList<Note> getNotes() {
		return notes;
	}

	/**
	 * 前打音を構成する音符リストをセットする。
	 *
	 * @param notes
	 *            セットする音符リスト
	 */
	public void setNotes(LinkedList<Note> notes) {
		this.notes = notes;
	}
}
