/**
 *
 */
package abc.score;

/**
 * K: や V: のフィールドに付与できるオプション(Clefやtranspositionなど)を表すインタフェース。
 *
 * @author CrapeMyrtle
 *
 */
public interface IInformationFieldClef {
	/**
	 * clef=～にあるオクターブ情報を取得する
	 *
	 * @return オクターブの変化量(+1/-1)。clef=～が定義されていない場合nullを返す。
	 */
	public Integer getClefOctave();

	/**
	 * clef=～にあるオクターブ情報をセットする
	 *
	 * @param octave
	 *            オクターブの変化量(+1/-1)。定義なしを表す場合nullをセットする。
	 */
	public void setClefOctave(Integer clefOctave);

	/**
	 * octave=～にあるオクターブ情報を取得する
	 *
	 * @return オクターブの変化量。octave=～が定義されていない場合nullを返す。
	 */
	public Integer getOctave();

	/**
	 * octave=～にあるオクターブ情報をセットする
	 *
	 * @param octave
	 *            オクターブの変化量。定義なしを表す場合nullをセットする。
	 */
	public void setOctave(Integer octave);

	/**
	 * transpose=～にある移調情報を取得する。
	 *
	 * @return 移調する量。半音=1とし、+1で半音上げる、-1で半音下げる。 transpose=～が定義されていない場合nullを返す。
	 */
	public Integer getTransposition();

	/**
	 * transpose=～にある移調情報をセットする。
	 *
	 * @param transposition
	 *            移調する量。半音=1とし、+1で半音上げる、-1で半音下げる。 定義なしを表す場合nullをセットする。
	 */
	public void setTransposition(Integer transposition);

	/**
	 * オクターブ情報をまとめた結果を取得する
	 * @return オクターブ情報
	 */
	public Integer getOctaveTotal();
}
