/**
 *
 */
package abc.score;

import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.Map.Entry;

/**
 * information field を保持するHashMap
 * @author CrapeMyrtle
 *
 */
@SuppressWarnings("serial")
public class MapInformationField extends LinkedHashMap<String, LinkedList<AInformationField>> implements Cloneable {

	/**
	 * コンストラクタ
	 */
	public MapInformationField() {
		super();
	}

	/**
	 * コンストラクタ。フィールド名、情報フィールドを1つ設定した状態で初期化する。
	 * @param name フィールド名
	 * @param field 情報フィールド
	 */
	public MapInformationField(String name, AInformationField field) {
		super();
		LinkedList<AInformationField> list = new LinkedList<AInformationField>();
		list.add(field);
		this.put(name, list);
	}

	/* (非 Javadoc)
	 * @see java.util.HashMap#clone()
	 */
	public MapInformationField clone() {
		MapInformationField map = new MapInformationField();
		for (Entry<String, LinkedList<AInformationField>> entry : this.entrySet()) {
			LinkedList<AInformationField> list = new LinkedList<AInformationField>();
			for (AInformationField field : entry.getValue()) {
				list.add((AInformationField)field.clone());
			}
			map.put(entry.getKey(), list);
		}
		return map;
	}

	/* (非 Javadoc)
	 * @see java.util.AbstractMap#toString()
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();

		sb.append("{");
		for (String key : this.keySet()) {
			sb.append(key + ":" + this.get(key).toString() + ",");
		}
		sb.append("}");

		return sb.toString();
	}

	/**
	 * 最新の(最後に追加された)情報フィールドを取得する。
	 * @param name フィールド名
	 * @return 情報フィールド
	 */
	public AInformationField getLast(String name) {
		LinkedList<AInformationField> list = this.get(name);
		if (list == null || list.size() == 0) {
			return null;
		}
		return list.getLast();
	}

}
