/**
 *
 */
package abc.score;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import abc.util.Fraction;

/**
 * 拍子を表すクラス
 *
 * @author CrapeMyrtle
 *
 */
public class Meter implements Cloneable {

	private static Pattern	PTN_INT			= Pattern.compile("^\\s*(\\d+)");
	private static Pattern	PTN_PLUS_INT	= Pattern
													.compile("^\\s*\\+\\s*(\\d+)");
	private static Pattern	PTN_BRACKET		= Pattern
													.compile("^\\s*\\(+(.*)\\)+$");
	private static Pattern	PTN_C			= Pattern.compile("C\\|?");

	private String			meter;
	private Fraction		length;

	/**
	 * コンストラクタ
	 */
	public Meter() {
		this.clear();
	}

	/**
	 * コンストラクタ。拍子を設定する。
	 *
	 * @param meter
	 *            設定する拍子
	 */
	public Meter(String meter) {
		this.setMeter(meter);
	}

	/**
	 * 拍子をクリアする
	 */
	public void clear() {
		this.meter = null;
		this.length = null;
	}

	/**
	 * 拍子が設定されてないかどうかを返す
	 * @return true: 拍子が設定されていない
	 */
	public boolean isNone() { return this.meter == null; }

	/**
	 * 拍子を取得する
	 *
	 * @return 拍子
	 */
	public String getMeter() {
		return this.meter;
	}

	/**
	 * 拍子を設定する。4/4、6/8など単純なもののみに対応しており、(2 + 3 + 2)/8 みたいなものには未対応。
	 *
	 * @param meter
	 *            拍子
	 */
	public void setMeter(String meter) {
		//TODO (2 + 3 + 2)/8 対応？ 面倒そう。。。
		Matcher m = PTN_C.matcher(meter);
		if (m.find()) {
			this.meter = m.group(0);
			this.length = new Fraction(1);
			return;
		}

		String[] numden = meter.split("/", 2);
		if (numden.length < 2) {
			this.clear();
		}

		m = PTN_INT.matcher(numden[1]);
		if (!m.find()) {
			this.clear();
		}
		int den = Integer.parseInt(m.group(1));

		m = PTN_BRACKET.matcher(numden[0]);
		if (m.find()) {
			numden[0] = m.group(1);
		}

		m = PTN_INT.matcher(numden[0]);
		if (!m.find()) {
			this.clear();
		}

		int num = Integer.parseInt(m.group(1));
		while (true) {
			numden[0] = numden[0].substring(m.end());
			m = PTN_PLUS_INT.matcher(numden[0]);
			if (m.find()) {
				num += Integer.parseInt(m.group(1));
			} else {
				break;
			}
		}

		this.meter = "" + num + "/" + den;
		this.length = new Fraction(num, den);
	}

	/**
	 * 長さを取得する
	 *
	 * @return 長さ
	 */
	public Fraction getLength() {
		return this.length;
	}

	/**
	 * 長さをセットする
	 * @param length セットする長さ
	 */
	public void setLength(Fraction length) {
		this.length = length;
	}

	/* (非 Javadoc)
	 * @see java.lang.Object#clone()
	 */
	public Meter clone() {
		return new Meter(this.meter);
	}

	/* (非 Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return this.meter + "=" + this.length.toString();
	}

}
