/**
 *
 */
package abc.score;

import abc.util.Fraction;

/**
 * テンポを表すクラス
 *
 * @author CrapeMyrtle
 *
 */
public class Tempo implements Cloneable {

	private Fraction	base;
	private int			count;

	/**
	 * コンストラクタ。1分間で1/4拍子120個で初期化する。
	 */
	public Tempo() {
		this.base = new Fraction(1, 4);
		this.count = 120;
	}

	/**
	 * コンストラクタ。ベースとなる音の長さ、音の個数を設定する。
	 *
	 * @param base
	 *            ベースとなる音の長さ
	 * @param count
	 *            音の個数
	 */
	public Tempo(Fraction base, int count) {
		this.base = base;
		this.count = count;
	}

	/*
	 * (非 Javadoc)
	 *
	 * @see java.lang.Object#clone()
	 */
	public Tempo clone() {
		return new Tempo(this.base, this.count);
	}

	/*
	 * (非 Javadoc)
	 *
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return (this.isNone() ? "none" : this.base.toString()) + "=" + this.count;
	}

	/**
	 * ベースとなる音の長さが設定されているかどうかを返す
	 *
	 * @return true: ベースとなる音の長さが設定されていない
	 */
	public boolean isNone() {
		return this.base == null;
	}

	/**
	 * ベースとなる音の長さを取得する
	 *
	 * @return base ベースとなる音の長さ
	 */
	public Fraction getBase() {
		return base;
	}

	/**
	 * ベースとなる音の長さをセットする
	 *
	 * @param base
	 *            セットする base
	 */
	public void setBase(Fraction base) {
		this.base = base;
	}

	/**
	 * 1分間の音の個数を取得する
	 *
	 * @return count 1分間の音の個数
	 */
	public int getCount() {
		return count;
	}

	/**
	 * 1分間の音の個数を設定する
	 *
	 * @param count
	 *            セットする count
	 */
	public void setCount(int count) {
		this.count = count;
	}

	/**
	 * ベースとなる音の長さを変更する
	 * @param base
	 */
	public void convertBase(Fraction base) {
		if (this.base == null) {
			this.base = base.clone();
			return;
		}

		if (this.base.equals(base)) {
			return;
		}

		this.count = (int)new Fraction(this.count).mul(this.base).div(base).getDouble();
		this.base = base.clone();
	}
}
