/**
 *
 */
package util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.Reader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.TreeMap;
import java.util.Map.Entry;

import abc.score.EDynamics;
import abc.util.Fraction;
import abc.util.TrackID;
import abc.util.TrackSrcDstID;

/**
 * コマンドライン引数の保持及び解析を行う。
 *
 * @author CrapeMyrtle
 *
 */
/**
 * @author CrapeMyrtle
 *
 */
public class Option {

	private Reader							in						= null;
	private PrintWriter						debug					= null;

	private String							strIn					= "-";
	private String							strOut					= "-";
	private String							strOutNow				= null;
	private String							strDebug				= "debug.txt";

	private HashMap<String, PrintWriter>	listOut					= new HashMap<String, PrintWriter>();
	private HashMap<String, String>			listOutTrack			= new HashMap<String, String>();

	private boolean							isKey2None				= false;
	private int								slurOutputType			= 0;
	private Fraction						slurLength				= new Fraction(1, 16);
	private int								accentCount				= 2;
	private Fraction						volumeChangeLength		= new Fraction(1, 8);
	private int								volumeChangeAmount		= 10;
	private Fraction						velocityChangeLength	= new Fraction(1, 8);
	private int								velocityChangeAmount	= 10;
	private Fraction						tempoChangeLength		= new Fraction(1, 8);
	private int								tempoChangeAmount		= 30;
	private Fraction						arpeggioLength			= new Fraction(1, 32);
	private int								appoggiaturaNum			= 2;
	private int								acciaccaturaNum			= 4;
	private boolean							isRelativeAppoggiatura	= false;
	private boolean							isAcciaccaturaBeforNote	= true;
	private Fraction						decorationLength		= new Fraction(1, 32);
	private boolean							isBarOutput				= false;
	private int								barAccentCount			= 0;
	private int								newlineStep				= 0;
	private boolean							isBarComment			= false;
	private TreeMap<EDynamics, Integer>		dynamics				= new TreeMap<EDynamics, Integer>();
	private boolean							isDynamicsVolume		= true;
	private boolean							isTrillFromUpper		= false;
	private Fraction						lengthScale				= new Fraction(1);

	private boolean							isHelp					= false;
	private boolean							isHelpDetail			= false;
	private boolean							isDebug					= false;
	private List<TrackID>					outList					= new ArrayList<TrackID>();
	private List<TrackSrcDstID>				combineList				= new ArrayList<TrackSrcDstID>();
	private List<TrackSrcDstID>				copyInformationList		= new ArrayList<TrackSrcDstID>();

	private boolean							isFixBugTempoChange		= false;

	private String							error					= null;

	private static final String				RES_HELP_FILENAME		= "res/help.txt";
	private static final String				RES_HELPDETAIL_FILENAME	= "res/help_detail.txt";
	private static final String				NL						= System.getProperty("line.separator");
	private static final Fraction			MIN_LENGTH				= new Fraction(1, 64);

	/**
	 * コンストラクタ
	 */
	public Option() {
		this.dynamics.put(EDynamics.PPP, 30);
		this.dynamics.put(EDynamics.PP, 45);
		this.dynamics.put(EDynamics.P, 60);
		this.dynamics.put(EDynamics.MP, 70);
		this.dynamics.put(EDynamics.MF, 90);
		this.dynamics.put(EDynamics.F, 105);
		this.dynamics.put(EDynamics.FF, 120);
		this.dynamics.put(EDynamics.FFF, 127);
	}

	/*
	 * (非 Javadoc)
	 *
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();

		sb.append("isKey2None=" + this.isKey2None + NL);
		sb.append("slurOutputType=" + this.slurOutputType + NL);
		sb.append("slurLength=" + this.slurLength + NL);
		sb.append("accentCount=" + this.accentCount + NL);
		sb.append("volumeChangeLength=" + this.volumeChangeLength + NL);
		sb.append("volumeChangeAmount=" + this.volumeChangeAmount + NL);
		sb.append("velocityChangeLength=" + this.velocityChangeLength + NL);
		sb.append("velocityChangeAmount=" + this.velocityChangeAmount + NL);
		sb.append("tempoChangeLength=" + this.tempoChangeLength + NL);
		sb.append("tempoChangeAmount=" + this.tempoChangeAmount + NL);
		sb.append("arpeggioLength=" + this.arpeggioLength + NL);
		sb.append("appoggiaturaNum=" + this.appoggiaturaNum + NL);
		sb.append("acciaccaturaNum=" + this.acciaccaturaNum + NL);
		sb.append("isRelativeAppoggiatura=" + this.isRelativeAppoggiatura + NL);
		sb.append("isAcciaccaturaBeforNote=" + this.isAcciaccaturaBeforNote + NL);
		sb.append("decorationLength=" + this.decorationLength + NL);
		sb.append("isBarOutput=" + this.isBarOutput + NL);
		sb.append("barAccentCount=" + this.barAccentCount + NL);
		sb.append("newlineStep=" + this.newlineStep + NL);
		sb.append("isBarComment=" + this.isBarComment + NL);
		for (Entry<EDynamics, Integer> entry : this.dynamics.entrySet()) {
			sb.append(entry.getKey().toString() + "=" + entry.getValue() + NL);
		}
		sb.append("isDynamicsVolume=" + this.isDynamicsVolume + NL);
		sb.append("isHelp=" + this.isHelp + NL);
		sb.append("isHelpDetail=" + this.isHelpDetail + NL);
		sb.append("isDebug=" + this.isDebug + NL);
		sb.append("outList=" + this.outList.toString() + NL);
		sb.append("combineList=" + this.combineList.toString() + NL);
		sb.append("copyInformationList=" + this.copyInformationList.toString() + NL);
		sb.append("error=" + this.error + NL);
		sb.append("isFixBugTempoChange=" + this.isFixBugTempoChange + NL);
		sb.append("listOutTrack=[" + NL);
		for (String key : this.listOutTrack.keySet()) {
			sb.append(" " + key + "=" + this.listOutTrack.get(key) + NL);
		}
		sb.append("]" + NL);

		return sb.toString();
	}

	/**
	 * 入力のReaderを取得する
	 *
	 * @return Reader
	 */
	public Reader getIn() {
		return in;
	}

	/**
	 * 入力のReaderをセットする
	 *
	 * @param in
	 *            入力Reader
	 */
	public void setIn(Reader in) {
		this.in = in;
	}

	/**
	 * 出力先のPrintWriterを取得する
	 *
	 * @return PrintWriter
	 * @throws IOException
	 */
	public PrintWriter getOut() throws IOException {
		PrintWriter pw = this.listOut.get(this.strOutNow);
		if (pw == null) {
			pw = new PrintWriter(new FileWriter(this.strOutNow));
			this.listOut.put(this.strOutNow, pw);
		}
		return pw;
	}

	/**
	 * 出力先をトラック毎の出力先にセットする。 トラック毎の出力先が設定されてなければデフォルトの出力先にセットする。
	 *
	 * @param tuneId
	 *            TuneID
	 * @param trackId
	 *            TrackID
	 */
	public void setOutTrack(String tuneId, String trackId) {
		String key = tuneId + ":" + trackId;
		String out = this.listOutTrack.get(key);
		if (out == null) {
			this.strOutNow = this.strOut;
		} else {
			this.strOutNow = out;
		}
	}

	/**
	 * 出力先をデフォルトの出力先にリセットする。
	 */
	public void resetOutTrack() {
		this.strOutNow = this.strOut;
	}

	/**
	 * 出力先のPrintWriterをセットする
	 *
	 * @param out
	 *            出力先のPrintWriter
	 */
	public void setOut(PrintWriter out) {
		String o = "";
		if (!this.strOut.equals("-")) {
			this.listOut.remove(this.strOut).close();
		}
		this.strOut = o;
		this.listOut.put(this.strOut, out);
	}

	/**
	 * トラック毎の出力先を追加する
	 *
	 * @param tuneId
	 *            TuneID
	 * @param trackId
	 *            TrackID
	 * @param out
	 *            出力先ファイル名
	 */
	public void addOutTrack(String tuneId, String trackId, String out) {
		String keyTrack = tuneId + ":" + trackId;
		if (!out.equals("-")) {
			File file = new File(out);
			out = file.getAbsolutePath();
			if (!this.listOut.containsKey(out)) {
				this.listOut.put(out, null);
			}
		}
		this.listOutTrack.put(keyTrack, out);
	}

	/**
	 * トラック毎の出力先を削除する
	 *
	 * @param tuneId
	 *            TuneID
	 * @param trackId
	 *            TrackID
	 */
	public void removeOutTrack(String tuneId, String trackId) {
		String keyTrack = tuneId + ":" + trackId;
		this.listOutTrack.remove(keyTrack);
	}

	/**
	 * キー(調、K:)を none(キー情報なし) にするかどうかを取得する
	 *
	 * @return true: キー情報なし
	 */
	public boolean isKey2None() {
		return isKey2None;
	}

	/**
	 * キー(調、K:)を none(キー情報なし) にするかどうかをセットする
	 *
	 * @param isKey2None
	 *            true: キー情報なしにセットする
	 */
	public void setKey2None(boolean isKey2None) {
		this.isKey2None = isKey2None;
	}

	/**
	 * スラーの出力タイプを取得する
	 *
	 * @return スラーの出力タイプ
	 *         <dl>
	 *         <dt>0</dt>
	 *         <dd>スラーを無視する</dd>
	 *         <dt>1</dt>
	 *         <dd>スラー内の音を伸ばす</dd>
	 *         <dt>2</dt>
	 *         <dd>スラー外の音を縮める</dd>
	 *         </dl>
	 */
	public int getSlurOutputType() {
		return slurOutputType;
	}

	/**
	 * スラーの出力タイプをセットする
	 *
	 * @param slurOutputType
	 *            セットするスラー出力タイプ(詳細は{@link Option#getSlurOutputType()})
	 */
	public void setSlurOutputType(int slurOutputType) {
		this.slurOutputType = slurOutputType;
	}

	/**
	 * スラー内/外の音を伸縮する長さを取得する
	 *
	 * @return 長さ
	 */
	public Fraction getSlurLength() {
		return slurLength;
	}

	/**
	 * スラー内/外の音を伸縮する長さをセットする
	 *
	 * @param slurLength
	 *            長さ セットする slurLength
	 */
	public void setSlurLength(Fraction slurLength) {
		this.slurLength = slurLength;
	}

	/**
	 * アクセント(!>!)の際に付与する臨時記号("~")の数を取得する
	 *
	 * @return 臨時記号の数
	 */
	public int getAccentCount() {
		return accentCount;
	}

	/**
	 * アクセント(!>!)の際に付与する臨時記号("~")の数をセットする
	 *
	 * @param accentCount
	 *            セットする臨時記号の数
	 */
	public void setAccentCount(int accentCount) {
		this.accentCount = accentCount;
	}

	/**
	 * 長前打音に対して除算する値を取得する。 長前打音の長さは以下のいずれかになる。({
	 * {@link Option#isRelativeAppoggiatura()}によりどちらの動作になるのか決定される)
	 * <ul>
	 * <li>親音の長さの 1/appoggiaturaNum 倍</li>
	 * <li>ABC譜上の長さの 1/appoggiaturaNum 倍</li>
	 * </ul>
	 *
	 * @return 音符の長さに対して除算する値
	 */
	public int getAppoggiaturaNum() {
		return appoggiaturaNum;
	}

	/**
	 * 長前打音に対して除算する値をセットする。
	 *
	 * @param appoggiaturaNum
	 *            セットするappoggiaturaNum
	 * @see Option#getAppoggiaturaNum()
	 */
	public void setAppoggiaturaNum(int appoggiaturaNum) {
		this.appoggiaturaNum = appoggiaturaNum;
	}

	/**
	 * 短前打音に対して除算する値を取得する。 短前打音の長さは以下のいずれかになる。({
	 * {@link Option#isRelativeAppoggiatura()()}によりどちらの動作になるのか決定される)
	 * <ul>
	 * <li>親音の長さの 1/appoggiaturaNum 倍</li>
	 * <li>ABC譜上の長さの 1/appoggiaturaNum 倍</li>
	 * </ul>
	 *
	 * @return 音符の長さに対して除算する値
	 */
	public int getAcciaccaturaNum() {
		return acciaccaturaNum;
	}

	/**
	 * 短前打音に対して除算する値をセットする。
	 *
	 * @param acciaccaturaNum
	 *            セットするacciaccaturaNum
	 */
	public void setAcciaccaturaNum(int acciaccaturaNum) {
		this.acciaccaturaNum = acciaccaturaNum;
	}

	/**
	 * 前打音の長さに対する倍率を設定する{@link Option#setAppoggiaturaNum(int)}や
	 * {@link Option#setAcciaccaturaNum(int)}について、 以下のいずれかの動作に決定する。
	 * <ol>
	 * <li>true: 親音の長さに対する倍率にする</li>
	 * <li>ABC譜上の長さに対する倍率にする</li>
	 * </ol>
	 *
	 * @return isRelativeappoggiatura
	 */
	public boolean isRelativeAppoggiatura() {
		return isRelativeAppoggiatura;
	}

	/**
	 * 前打音の長さに対する倍率を設定する{@link Option#setAppoggiaturaNum(int)}や
	 * {@link Option#setAcciaccaturaNum(int)}の 動作を決定する。
	 *
	 * @param isRelativeAppoggiatura
	 *            セットするisRelativeAppoggiatura
	 * @see Option#isRelativeAppoggiatura()
	 */
	public void setRelativeAppoggiatura(boolean isRelativeAppoggiatura) {
		this.isRelativeAppoggiatura = isRelativeAppoggiatura;
	}

	/**
	 * 短前打音を直前の音符の末尾に付与するかどうかを取得する
	 *
	 * @return true: 短前打音を直前の音符の末尾に追加する
	 */
	public boolean isAcciaccaturaBeforNote() {
		return isAcciaccaturaBeforNote;
	}

	/**
	 * 短前打音を直前の音符の末尾に付与するかどうかをセットする
	 *
	 * @param isAcciaccaturaBeforNote
	 *            セットするisAcciaccaturaBeforNote
	 */
	public void setAcciaccaturaBeforNote(boolean isAcciaccaturaBeforNote) {
		this.isAcciaccaturaBeforNote = isAcciaccaturaBeforNote;
	}

	/**
	 * 装飾記号(トリル、ターン、モルデント等)の長さを取得する
	 *
	 * @return 装飾記号の長さ
	 */
	public Fraction getDecorationLength() {
		return decorationLength;
	}

	/**
	 * 装飾記号(トリル、ターン、モルデント等)の長さをセットする
	 *
	 * @param decorationLength
	 *            セットする装飾記号の長さ
	 */
	public void setDecorationLength(Fraction decorationLength) {
		this.decorationLength = decorationLength;
	}

	/**
	 * 強弱法をVolume変化で表現するかどうかを取得する
	 *
	 * @return true: 強弱法をVolume変化で表現する。false: 強弱法をVelocity変化で表現する。
	 */
	public boolean isDynamicsVolume() {
		return isDynamicsVolume;
	}

	/**
	 * 強弱法をVolume変化で表現するかどうかをセットする
	 *
	 * @param isDynamicsVolume
	 *            true: 強弱法をVolume変化で表現する。false: 強弱法をVelocity変化で表現する。
	 */
	public void setDynamicsVolume(boolean isDynamicsVolume) {
		this.isDynamicsVolume = isDynamicsVolume;
	}

	/**
	 * 小節線を出力するかどうかを取得する
	 *
	 * @return true: 小節線を出力する
	 */
	public boolean isBarOutput() {
		return isBarOutput;
	}

	/**
	 * 小節線を出力するかどうかをセットする
	 *
	 * @param isBarOutput
	 *            true: 小節線を出力する
	 */
	public void setBarOutput(boolean isBarOutput) {
		this.isBarOutput = isBarOutput;
	}

	/**
	 * 小節の最初の音に付与するアクセント(~)の個数を取得する
	 *
	 * @return アクセントの個数
	 */
	public int getBarAccentCount() {
		return barAccentCount;
	}

	/**
	 * 小節の最初の音に付与するアクセント(~)の個数をセットする
	 *
	 * @param barAccentCount
	 *            セットするアクセントの個数
	 */
	public void setBarAccentCount(int barAccentCount) {
		this.barAccentCount = barAccentCount;
	}

	/**
	 * 改行を行う小節数を取得する
	 *
	 * @return 改行を行う小節数
	 */
	public int getNewlineStep() {
		return newlineStep;
	}

	/**
	 * 改行を行う小節数をセットする
	 *
	 * @param newlineStep
	 *            改行を行う小節数
	 */
	public void setNewlineStep(int newlineStep) {
		this.newlineStep = newlineStep;
	}

	/**
	 * 小節番号をコメントとして出力するかどうかを取得する
	 *
	 * @return true: 小節番号をコメントとして出力する
	 */
	public boolean isBarComment() {
		return isBarComment;
	}

	/**
	 * 小節番号をコメントとして出力するかどうかを
	 *
	 * @param isBarComment
	 *            true: 小節番号をコメントとして出力する
	 */
	public void setBarComment(boolean isBarComment) {
		this.isBarComment = isBarComment;
	}

	/**
	 * クレッシェンドなどによるTune全体のボリューム変化を出力する間隔を取得する
	 *
	 * @return 出力間隔
	 */
	public Fraction getVolumeChangeLength() {
		return volumeChangeLength;
	}

	/**
	 * クレッシェンドなどによるTune全体のボリューム変化を出力する間隔をセットする
	 *
	 * @param volumeChangeLength
	 *            セットする出力間隔
	 */
	public void setVolumeChangeLength(Fraction volumeChangeLength) {
		this.volumeChangeLength = volumeChangeLength;
	}

	/**
	 * クレッシェンドなどによるTune全体のボリューム変化量を取得する
	 *
	 * @return ボリューム変化の解像度
	 */
	public int getVolumeChangeAmount() {
		return volumeChangeAmount;
	}

	/**
	 * クレッシェンドなどによるTune全体のボリューム変化量
	 *
	 * @param volumeChangeAmount
	 *            セットするボリューム変化量
	 */
	public void setVolumeChangeAmount(int volumeChangeAmount) {
		this.volumeChangeAmount = volumeChangeAmount;
	}

	/**
	 * クレッシェンドなどによるTrack毎のヴェロシティ変化を出力する間隔を取得する
	 *
	 * @return 出力間隔
	 */
	public Fraction getVelocityChangeLength() {
		return velocityChangeLength;
	}

	/**
	 * クレッシェンドなどによるTrack毎のヴェロシティ変化を出力する間隔をセットする
	 *
	 * @param velocityChangeLength
	 *            セットする出力間隔
	 */
	public void setVelocityChangeLength(Fraction velocityChangeLength) {
		this.velocityChangeLength = velocityChangeLength;
	}

	/**
	 * クレッシェンドなどによるTrack毎のヴェロシティ変化量を取得する
	 *
	 * @return ヴェロシティ変化量
	 */
	public int getVelocityChangeAmount() {
		return velocityChangeAmount;
	}

	/**
	 * クレッシェンドなどによるTrack毎のヴェロシティ変化量をセットする
	 *
	 * @param velocityChangeAmount
	 *            セットするヴェロシティ変化量
	 */
	public void setVelocityChangeAmount(int velocityChangeAmount) {
		this.velocityChangeAmount = velocityChangeAmount;
	}

	/**
	 * リタルダンドなどによるテンポ変化を出力する間隔を取得する
	 *
	 * @return 出力間隔
	 */
	public Fraction getTempoChangeLength() {
		return tempoChangeLength;
	}

	/**
	 * リタルダンドなどによるテンポ変化を出力する間隔をセットする
	 *
	 * @param tempoChangeLength
	 *            セットする出力間隔
	 */
	public void setTempoChangeLength(Fraction tempoChangeLength) {
		this.tempoChangeLength = tempoChangeLength;
	}

	/**
	 * リタルダンドなどによるテンポ変化量を取得する
	 *
	 * @return テンポ変化量
	 */
	public int getTempoChangeAmount() {
		return tempoChangeAmount;
	}

	/**
	 * リタルダンドなどによるテンポ変化量をセットする
	 *
	 * @param tempoChangeAmount
	 *            セットするテンポ変化量
	 */
	public void setTempoChangeAmount(int tempoChangeAmount) {
		this.tempoChangeAmount = tempoChangeAmount;
	}

	/**
	 * アルペッジョの音符追加間隔を取得する
	 *
	 * @return アルペッジョの音符追加間隔
	 */
	public Fraction getArpeggioLength() {
		return arpeggioLength;
	}

	/**
	 * アルペッジョの音符追加間隔をセットする
	 *
	 * @param arpeggioLength
	 *            セットするアルペッジョの音符追加間隔
	 */
	public void setArpeggioLength(Fraction arpeggioLength) {
		this.arpeggioLength = arpeggioLength;
	}

	/**
	 * 強弱法の表す値を取得する
	 *
	 * @param dynamic
	 *            強弱法
	 * @return 値
	 */
	public int getDynamicsValue(EDynamics dynamic) {
		return this.dynamics.get(dynamic);
	}

	/**
	 * 強弱法の表す値をセットする
	 *
	 * @param dynamic
	 *            強弱法
	 * @param value
	 *            値
	 */
	public void setDynamicsValue(EDynamics dynamic, int value) {
		this.dynamics.put(dynamic, value);
	}

	/**
	 * Note Unit Length(L:)にかける倍率を取得する
	 * @return Note Unit Length(L:)にかける倍率
	 */
	public Fraction getLengthScale() {
		return lengthScale;
	}

	/**
	 * Note Unit Length(L:)にかける倍率をセットする
	 * @param lengthScale セットするNote Unit Length(L:)にかける倍率
	 */
	public void setLengthScale(Fraction lengthScale) {
		this.lengthScale = lengthScale;
	}

	/**
	 * トリルを2度上から開始するかどうか
	 *
	 * @return true: トリルを2度上から開始。 false: トリルを1度から開始する。
	 */
	public boolean isTrillFromUpper() {
		return isTrillFromUpper;
	}

	/**
	 * トリルを2度上から開始するかどうをセットする
	 *
	 * @param isTrillFromUpper
	 *            true: トリルを2度上から開始。 false: トリルを1度から開始する。
	 */
	public void setTrillFromUpper(boolean isTrillFromUpper) {
		this.isTrillFromUpper = isTrillFromUpper;
	}

	/**
	 * ヘルプ表示かどうかを取得する
	 *
	 * @return true: ヘルプ表示
	 */
	public boolean isHelp() {
		return isHelp;
	}

	/**
	 * ヘルプ表示かどうかを設定する
	 *
	 * @param isHelp
	 *            ヘルプ表示
	 */
	public void setHelp(boolean isHelp) {
		this.isHelp = isHelp;
	}

	/**
	 * 詳細なヘルプ表示かどうかを取得する
	 *
	 * @return true: ヘルプ表示
	 */
	public boolean isHelpDetail() {
		return isHelpDetail;
	}

	/**
	 * 詳細なヘルプ表示かどうかを設定する
	 *
	 * @param isHelpDetail
	 *            ヘルプ表示
	 */
	public void setHelpDetail(boolean isHelpDetail) {
		this.isHelpDetail = isHelpDetail;
	}

	/**
	 * 出力リストを取得する
	 *
	 * @return 出力リスト
	 */
	public List<TrackID> getOutList() {
		return outList;
	}

	/**
	 * Trackの合成リストを取得する
	 *
	 * @return 合成リスト
	 */
	public List<TrackSrcDstID> getCombineList() {
		return combineList;
	}

	/**
	 * Trackの情報コピーリストを取得する
	 *
	 * @return 合成リスト
	 */
	public List<TrackSrcDstID> getCopyInformationList() {
		return copyInformationList;
	}

	/**
	 * MoE-ABC側のバグ(テンポチェンジ)に対応するかどうかを取得する
	 *
	 * @return true: 対応する
	 */
	public boolean isFixBugTempoChange() {
		return isFixBugTempoChange;
	}

	/**
	 * MoE-ABC側のバグ(テンポチェンジ)に対応するかどうかをセットする
	 *
	 * @param isFixBugTempoChange
	 *            true: 対応する
	 */
	public void setFixBugTempoChange(boolean isFixBugTempoChange) {
		this.isFixBugTempoChange = isFixBugTempoChange;
	}

	/**
	 * デバッグ情報を出力するかどうかを取得する
	 *
	 * @return true: デバッグ情報を出力
	 */
	public boolean isDebug() {
		return isDebug;
	}

	/**
	 * デバッグ情報を出力するかどうかをセットする
	 *
	 * @param isDebug
	 *            true: デバッグ情報を出力する
	 */
	public void setDebug(boolean isDebug) {
		this.isDebug = isDebug;
	}

	/**
	 * デバッグ出力を取得する
	 *
	 * @return デバッグ出力
	 */
	public PrintWriter getDebug() {
		return debug;
	}

	/**
	 * エラー情報を取得する
	 *
	 * @return エラー情報。なければnullを返す。
	 */
	public String getError() {
		return error;
	}

	/**
	 * コマンドライン引数を解析する
	 *
	 * @param args
	 *            コマンドライン引数
	 */
	public void parseOpt(String[] args) {
		String arg = "";
		int i = -1;
		try {
			StringBuffer error = new StringBuffer();

			for (i = 0; i < args.length; ++i) {
				arg = args[i];
				switch (arg) {
				case "-k":
				case "--key":
					this.isKey2None = false;
					break;
				case "-nk":
				case "--no-key":
					this.isKey2None = true;
					break;
				case "-s":
				case "--slur":
					this.slurOutputType = Integer.parseInt(args[++i]);
					if (0 <= this.slurOutputType && this.slurOutputType <= 2) {
					} else {
						error.append(arg + "は0, 1, 2を設定して下さい" + NL);
						this.isHelp = true;
						this.isHelpDetail = true;
					}
					break;
				case "-sl":
				case "--slur-length":
					this.slurLength = Fraction.parseFraction(args[++i]);
					if (this.slurLength.compareTo(Fraction.ZERO) <= 0) {
						error.append(arg + "は0より大きい数を設定して下さい" + NL);
						this.isHelp = true;
						this.isHelpDetail = true;
					}
					break;
				case "-a":
				case "--accent-count":
					this.accentCount = Integer.parseInt(args[++i]);
					if (this.accentCount < 1) {
						error.append(arg + "は1以上を設定して下さい" + NL);
						this.isHelp = true;
						this.isHelpDetail = true;
					}
					break;
				case "-i":
				case "--input":
					this.strIn = args[++i];
					break;
				case "-o":
				case "--output":
					this.strOut = args[++i];
					break;

				case "-ot":
				case "--out-track": {
					String[] idTuneTrack = args[++i].split(":", 2);
					String out = args[++i];
					this.addOutTrack(idTuneTrack[0], idTuneTrack[1], out);
				}
					break;

				case "-t":
				case "--track":
					String[] idTuneTrack = args[++i].split(":", 2);
					this.outList.add(new TrackID(idTuneTrack[0], idTuneTrack[1]));
					break;
				case "-c":
				case "--combine": {
					String[] tracks = args[++i].split(",", 2);
					String[] idTuneTrackSrc = tracks[0].split(":", 2);
					String[] idTuneTrackDst = tracks[1].split(":", 2);
					this.combineList.add(new TrackSrcDstID(idTuneTrackSrc[0], idTuneTrackSrc[1],
							idTuneTrackDst[0], idTuneTrackDst[1]));
					break;
				}
				case "-ci":
				case "--copy-information": {
					String[] tracks = args[++i].split(",", 2);
					String[] idTuneTrackSrc = tracks[0].split(":", 2);
					String[] idTuneTrackDst = tracks[1].split(":", 2);
					this.copyInformationList.add(new TrackSrcDstID(idTuneTrackSrc[0],
							idTuneTrackSrc[1], idTuneTrackDst[0], idTuneTrackDst[1]));
					break;
				}
				case "-Vl":
				case "--volume-change-length":
					this.volumeChangeLength = Fraction.parseFraction(args[++i]);
					if (this.volumeChangeLength.compareTo(MIN_LENGTH) < 0) {
						error.append(arg + "は" + MIN_LENGTH + "以上を設定して下さい" + NL);
						this.isHelp = true;
						this.isHelpDetail = true;
					}
					break;
				case "-Va":
				case "--volume-change-amount":
					this.volumeChangeAmount = Integer.parseInt(args[++i]);
					if (this.volumeChangeAmount < 1) {
						error.append(arg + "は1以上を設定して下さい" + NL);
						this.isHelp = true;
						this.isHelpDetail = true;
					}
					break;
				case "-vl":
				case "--velocity-change-length":
					this.velocityChangeLength = Fraction.parseFraction(args[++i]);
					if (this.velocityChangeLength.compareTo(MIN_LENGTH) < 0) {
						error.append(arg + "は" + MIN_LENGTH + "以上を設定して下さい" + NL);
						this.isHelp = true;
						this.isHelpDetail = true;
					}
					break;
				case "-va":
				case "--velocity-change-amount":
					this.velocityChangeAmount = Integer.parseInt(args[++i]);
					if (this.velocityChangeAmount < 1) {
						error.append(arg + "は1以上を設定して下さい" + NL);
						this.isHelp = true;
						this.isHelpDetail = true;
					}
					break;
				case "-tl":
				case "--tempo-change-length":
					this.tempoChangeLength = Fraction.parseFraction(args[++i]);
					if (this.tempoChangeLength.compareTo(MIN_LENGTH) < 0) {
						error.append(arg + "は" + MIN_LENGTH + "以上を設定して下さい" + NL);
						this.isHelp = true;
						this.isHelpDetail = true;
					}
					break;
				case "-ta":
				case "--tempo-change-amount":
					this.tempoChangeAmount = Integer.parseInt(args[++i]);
					if (this.tempoChangeAmount < 1) {
						error.append(arg + "は1以上を設定して下さい" + NL);
						this.isHelp = true;
						this.isHelpDetail = true;
					}
					break;
				case "-Al":
				case "--arpeggio-length":
					this.arpeggioLength = Fraction.parseFraction(args[++i]);
					if (this.arpeggioLength.compareTo(MIN_LENGTH) < 0) {
						error.append(arg + "は" + MIN_LENGTH + "以上を設定して下さい" + NL);
						this.isHelp = true;
						this.isHelpDetail = true;
					}
					break;
				case "-ar":
				case "--appo-relative":
					this.isRelativeAppoggiatura = true;
					break;
				case "-nar":
				case "--no-appo-relative":
					this.isRelativeAppoggiatura = false;
					break;
				case "-ap":
				case "--appo-num":
					this.appoggiaturaNum = Integer.parseInt(args[++i]);
					if (this.appoggiaturaNum < 1) {
						error.append(arg + "は1以上を設定して下さい" + NL);
						this.isHelp = true;
						this.isHelpDetail = true;
					}
					break;
				case "-ac":
				case "--acci-num":
					this.acciaccaturaNum = Integer.parseInt(args[++i]);
					if (this.acciaccaturaNum < 1) {
						error.append(arg + "は1以上を設定して下さい" + NL);
						this.isHelp = true;
						this.isHelpDetail = true;
					}
					break;
				case "-abn":
				case "--acci-before-note":
					this.isAcciaccaturaBeforNote = true;
					break;
				case "-aan":
				case "--acci-after-note":
					this.isAcciaccaturaBeforNote = false;
					break;
				case "-dl":
				case "--decoration-length":
					this.decorationLength = Fraction.parseFraction(args[++i]);
					// if (this.decorationLength.compareTo(new Fraction(5, 256))
					// < 0) {
					// error.append(arg + "は" + new Fraction(5, 256) +
					// "以上を設定して下さい" + NL);
					if (this.decorationLength.compareTo(Option.MIN_LENGTH) < 0) {
						error.append(arg + "は" + Option.MIN_LENGTH + "以上を設定して下さい" + NL);
						this.isHelp = true;
						this.isHelpDetail = true;
					}
					break;
				case "-ls":
				case "--length-scale":
					this.lengthScale = Fraction.parseFraction(args[++i]);
					break;
				case "-tu":
				case "--trill-from-upper":
					this.isTrillFromUpper = true;
					break;
				case "-ts":
				case "--trill-from-same":
					this.isTrillFromUpper = false;
					break;
				case "-ppp":
				case "-pp":
				case "-p":
				case "-mp":
				case "-mf":
				case "-f":
				case "-ff":
				case "-fff":
					EDynamics dyn = EDynamics.valueOf(args[i].substring(1).toUpperCase());
					int val = Integer.parseInt(args[++i]);
					this.dynamics.put(dyn, val);
					if (val < 1) {
						error.append(arg + "は1以上を設定して下さい" + NL);
						this.isHelp = true;
						this.isHelpDetail = true;
					}
					break;
				case "-dV":
				case "--dynamics-Volume":
					this.isDynamicsVolume = true;
					break;
				case "-dv":
				case "--dynamics-velocity":
					this.isDynamicsVolume = false;
					break;
				case "-b":
				case "--bar":
					this.isBarOutput = true;
					break;
				case "-nb":
				case "--no-bar":
					this.isBarOutput = false;
					break;
				case "-ba":
				case "--bar-accent":
					this.barAccentCount = Integer.parseInt(args[++i]);
					if (this.barAccentCount < 0) {
						error.append(arg + "は0以上を設定して下さい" + NL);
						this.isHelp = true;
						this.isHelpDetail = true;
					}
					break;
				case "-bnl":
				case "--bar-new-line":
					this.newlineStep = Integer.parseInt(args[++i]);
					if (this.newlineStep < 0) {
						error.append(arg + "は0以上を設定して下さい" + NL);
						this.isHelp = true;
						this.isHelpDetail = true;
					}
					break;
				case "-bc":
				case "--bar-comment":
					this.isBarComment = true;
					break;
				case "-nbc":
				case "--no-bar-comment":
					this.isBarComment = false;
					break;

				case "-fb-tc":
				case "--fix-bug-tempo-change":
					this.isFixBugTempoChange = true;
					break;
				case "-d":
				case "--debug":
					this.isDebug = true;
					break;

				case "-h":
				case "--help":
					this.isHelp = true;
					break;
				case "-hd":
				case "--help-detail":
					this.isHelp = true;
					this.isHelpDetail = true;
					break;
				default:
					error.append("無効なオプションです: \"" + args[i] + "\"" + NL);
					this.isHelp = true;
					this.isHelpDetail = true;
					break;
				}
			}

			if (this.in == null) {
				if (this.strIn.equals("-")) {
					this.in = new InputStreamReader(System.in);
				} else {
					this.in = new FileReader(this.strIn);
				}
			}

			this.listOut.put("-", new PrintWriter(System.out, true));
			if (!this.strOut.equals("-")) {
				File file = new File(this.strOut);
				this.strOut = file.getAbsolutePath();
			}
			this.strOutNow = this.strOut;

			if (this.debug == null && this.isDebug) {
				this.debug = new PrintWriter(new FileWriter(this.strDebug));
			}

			if (error.length() > 0) {
				this.error = error.toString();
			}
		} catch (ArrayIndexOutOfBoundsException e) {
			this.error = "" + i + "個目の引数(" + arg + ") ArrayIndexOutOfBoundsException: "
					+ e.getMessage();
		} catch (FileNotFoundException e) {
			this.error = "" + i + "個目の引数(" + arg + ") FileNotFoundException: " + e.getMessage();
		} catch (IOException e) {
			this.error = "" + i + "個目の引数(" + arg + ") IOException: " + e.getMessage();
		} catch (NumberFormatException e) {
			this.error = "" + i + "個目の引数(" + arg + ")に無効な数字が与えられました: " + e.getMessage();
		}
	}

	/**
	 * ヘルプ文字列を取得する
	 *
	 * @return ヘルプ文字列
	 * @throws IOException
	 */
	public String help() throws IOException {
		StringBuffer sb = new StringBuffer();

		InputStream is = this.getClass().getClassLoader().getResourceAsStream(RES_HELP_FILENAME);
		BufferedReader br = new BufferedReader(new InputStreamReader(is != null
				? is
				: new FileInputStream(RES_HELP_FILENAME), "utf-8"));

		String line;
		while ((line = br.readLine()) != null) {
			sb.append(line);
			sb.append(NL);
		}

		if (this.isHelpDetail) {
			is = this.getClass().getClassLoader().getResourceAsStream(RES_HELPDETAIL_FILENAME);
			br = new BufferedReader(new InputStreamReader(is != null ? is : new FileInputStream(
					RES_HELPDETAIL_FILENAME), "utf-8"));

			sb.append(NL);
			while ((line = br.readLine()) != null) {
				sb.append(line);
				sb.append(NL);
			}
		}

		return sb.toString();
	}

	/**
	 * 入出力のリソースをcloseする
	 *
	 * @throws IOException
	 */
	public void closeResource() throws IOException {
		if (this.in != null) {
			if (!this.strIn.equals("-")) {
				this.in.close();
			}
			this.in = null;
			this.strIn = null;
		}

		this.strOut = null;

		for (String key : this.listOut.keySet()) {
			if (!key.equals("-")) {
				PrintWriter pw = this.listOut.get(key);
				if (pw != null) {
					pw.close();
					pw = null;
				}
			}
		}

		if (this.debug != null) {
			this.debug.close();
			this.debug = null;
		}
	}
}
